@@ -1,5 +1,3 @@
-# Interchange6::Cart - Interchange6 cart class
-
 package Interchange6::Cart;
 
 =head1 NAME 
@@ -8,14 +6,10 @@ Interchange6::Cart - Cart class for Interchange6 Shop Machine
 
 =cut
 
-use strict;
 use Carp;
-use DateTime;
-use Interchange6::Cart::Product;
-use Scalar::Util 'blessed';
-use Try::Tiny;
-use Types::Common::String qw/NonEmptyStr/;
-use Types::Standard qw/ArrayRef InstanceOf Str/;
+use Interchange6::Types -types;
+use Module::Runtime 'use_module';
+use Safe::Isa;
 
 use Moo;
 use MooX::HandlesVia;
@@ -98,13 +92,14 @@ has products => (
     # rwp allows us to clear out products in seed via _set_products
     # without disturbing what subclasses might expect of clear
     is  => 'rwp',
-    isa => ArrayRef [ InstanceOf ['Interchange6::Cart::Product'] ],
+    isa => ArrayRef [ CartProduct ],
     default     => sub { [] },
     handles_via => 'Array',
     handles     => {
         clear          => 'clear',
         count          => 'count',
         is_empty       => 'is_empty',
+        product_first  => 'first',
         product_get    => 'get',
         product_index  => 'first_index',
         products_array => 'elements',
@@ -115,6 +110,19 @@ has products => (
     init_arg => undef,
 );
 
+=head2 product_class
+
+To allow use of a subclassed L<Interchange6::Cart::Product>. Defaults to
+C<Interchange6::Cart::Product>.
+
+=cut
+
+has product_class => (
+    is      => 'ro',
+    isa     => Str,
+    default => 'Interchange6::Cart::Product',
+);
+
 =head2 sessions_id
 
 The session ID for the cart.
@@ -274,37 +282,14 @@ sub add {
     my $product = $_[0];
     my ( $index, $oldproduct, $update );
 
-    if ( !defined $product ) {
-        die "undefined argument passed to add";
-    }
-    elsif ( blessed($product) ) {
-        die "product argument is not an Interchange6::Cart::Product"
-          unless ( $product->isa('Interchange6::Cart::Product') );
-    }
-    else {
+    croak "undefined argument passed to add" unless defined $product;
 
-        my %args;
+    $product = use_module( $self->product_class )->new(@_)
+      unless $product->$_isa( $self->product_class );
 
-        if ( @_ % 2 ) {
-            if ( ref($product) eq 'HASH' ) {
-
-                # copy args
-                %args = %{$product};
-            }
-            else {
-                die "argument to add should be hash or hashref";
-            }
-        }
-        else {
-
-            %args = @_;
-        }
-
-        $product = 'Interchange6::Cart::Product'->new(%args);
-    }
-
-   # cart may already contain an product with the same sku
-   # if so then we add quantity to existing product otherwise we add new product
+    # Cart may already contain an product with the same sku.
+    # If so then we add quantity to existing product otherwise we add new
+    # product.
 
     $index = $self->product_index( sub { $_->sku eq $product->sku } );
 
@@ -333,7 +318,7 @@ sub add {
 
 =head2 find
 
-Searches for an cart product with the given SKU.
+Searches for a cart product with the given SKU.
 Returns cart product in case of sucess or undef on failure.
 
   if ($product = $cart->find(9780977920174)) {
@@ -344,14 +329,7 @@ Returns cart product in case of sucess or undef on failure.
 
 sub find {
     my ( $self, $sku ) = @_;
-
-    for my $cartproduct ( $self->products_array ) {
-        if ( $sku eq $cartproduct->sku ) {
-            return $cartproduct;
-        }
-    }
-
-    return undef;
+    $self->product_first( sub { $sku eq $_->sku } );
 }
 
 =head2 has_subtotal
@@ -395,15 +373,15 @@ Remove product from the cart. Takes SKU of product to identify the product.
 sub remove {
     my ( $self, $arg ) = @_;
 
-    die "no argument passed to remove" unless defined $arg;
+    croak "no argument passed to remove" unless defined $arg;
 
     my $index = $self->product_index( sub { $_->sku eq $arg } );
 
-    die "sku $arg not found in cart" unless $index >= 0;
+    croak "sku $arg not found in cart" unless $index >= 0;
 
     # remove product from our array
     my $ret = $self->product_delete($index);
-    die "remove sku $arg failed" unless defined $ret;
+    croak "remove sku $arg failed" unless defined $ret;
 
     return $ret;
 }
@@ -430,21 +408,16 @@ On success returns L</products>.
 sub seed {
     my ( $self, $product_ref ) = @_;
 
-    die "argument to seed must be an array reference"
+    croak "argument to seed must be an array reference"
       unless ref($product_ref) eq 'ARRAY';
 
-    # we can't use ->clear since subclasses might wrap that to do
-    # interesting things like remove products from cart in database
-    $self->_set_products([]);
+    my $product_class = use_module( $self->product_class );
 
     my @products;
     for my $args ( @{$product_ref} ) {
-        push @products, Interchange6::Cart::Product->new($args);
+        push @products, $product_class->new($args);
     }
-
-    # if we got here then all products are good so put them in the cart
-    $self->product_push(@products);
-
+    $self->_set_products( \@products );
     return $self->products;
 }
 
@@ -473,16 +446,12 @@ sub update {
         $sku = shift @args;
         $qty = shift @args;
 
-        die "sku not defined in arg to update" unless defined $sku;
+        croak "sku not defined in arg to update" unless defined $sku;
 
-        die "quantity not supplied as arg to update for sku $sku"
-          unless defined $qty;
-
-        die "quantity must be a positive integer or zero"
-          unless $qty =~ /^\d+$/;
+        defined($qty) or croak "quantity argument to update must be defined";
 
         unless ( $product = $self->find($sku) ) {
-            die "Product for $sku not found in cart.";
+            croak "Product for $sku not found in cart.";
         }
 
         if ( $qty == 0 ) {
