@@ -1,11 +1,6 @@
-# Interchange6::Cart::Product - Interchange6 cart product class
-
 package Interchange6::Cart::Product;
 
-use strict;
-use Types::Common::Numeric qw/PositiveInt PositiveOrZeroNum/;
-use Types::Common::String qw/NonEmptyStr/;
-use Types::Standard qw/Defined HashRef HasMethods InstanceOf Int Maybe Num Str Undef/;
+use Interchange6::Types -types;
 
 use Moo;
 use MooX::HandlesVia;
@@ -54,7 +49,7 @@ A reference to the Cart object that this Cart::Product belongs to.
 
 has cart => (
     is      => 'ro',
-    isa     =>  Maybe[InstanceOf ['Interchange6::Cart']],
+    isa     =>  Maybe[Cart],
     default => undef,
     writer  => 'set_cart',
     weak_ref => 1,
@@ -150,13 +145,22 @@ than zero. Default for quantity is 1.
 =cut
 
 has quantity => (
-    is      => 'ro',
-    isa     => PositiveInt,
+    is => 'ro',
+    # https://github.com/interchange/Interchange6/issues/28
+    # Tupe::Tiny::XS sometimes incorrectly passes Int assertion for
+    # non-integer values such as 2.3 so we can't just do:
+    #    isa => PositiveInt
+    # but if we stringify the value then things work as expected. Huh?
+    # Also prevent uninitialized warning in case value is undef.
+    isa => sub {
+        no warnings 'uninitialized';
+        PositiveInt->assert_valid("$_[0]");
+    },
     default => 1,
     writer  => 'set_quantity',
 );
 
-after quantity => sub {
+after set_quantity => sub {
     my $self = shift;
     $self->clear_subtotal;
     $self->clear_total;
@@ -332,7 +336,7 @@ sub is_canonical {
 # after cost changes we need to clear the cart subtotal/total
 # our own total is handled by the Costs role
 
-after 'clear_costs', 'cost_set', 'cost_push', 'set_quantity' => sub {
+after 'clear_costs', 'cost_set', 'apply_cost', 'set_quantity' => sub {
     my $self = shift;
     if ( $self->cart ) {
         $self->cart->clear_subtotal;
